@@ -21,22 +21,20 @@
 #' }
 #' last two fields are available if proper values for calculation of the statistics were provided.
 #' 
-regcoeffs = function(coeffs, ci.coeffs = NULL, ci.alpha = 0.1)
-{   
-
-   regcoeffs = list(values = coeffs)
-   if (!is.null(ci.coeffs))
-   {
+regcoeffs = function(coeffs, ci.coeffs = NULL, ci.alpha = 0.1) {   
+   regcoeffs = list()
+   regcoeffs$values = coeffs
+   if (!is.null(ci.coeffs)) {
       stat = regcoeffs.getStat(coeffs, ci.coeffs, ci.alpha)
       regcoeffs$ci = stat$ci
       regcoeffs$t.values = stat$t.values
       regcoeffs$p.values = stat$p.values
       regcoeffs$alpha = ci.alpha
    }   
+   
    regcoeffs$call = match.call()
    
    class(regcoeffs) = "regcoeffs"
-   
    regcoeffs
 }
 
@@ -46,7 +44,7 @@ regcoeffs = function(coeffs, ci.coeffs = NULL, ci.alpha = 0.1)
 #' calculates confidence intervals and t-test based p-values for 
 #' regression coefficients based on jack-knifing procedure
 #' 
-#' @param obj
+#' @param coeffs.values
 #' regression coefficients array for a model
 #' @param ci.coeffs
 #' array with regression coefficients for calculation of condifence intervals
@@ -57,23 +55,26 @@ regcoeffs = function(coeffs, ci.coeffs = NULL, ci.alpha = 0.1)
 #' a list with statistics (\code{$ci} - array with confidence intervals, 
 #' \code{$p.values} - array with p-values, \code{$t.values} - array with t-values)
 #' 
-regcoeffs.getStat = function(obj, ci.coeffs, ci.alpha = 0.1)
-{
-   s = dim(ci.coeffs)
-   nvar = s[1]
-   ncomp = s[2]
-   ny = s[3]
-   nobj = s[4]
+regcoeffs.getStat = function(coeffs.values, ci.coeffs, ci.alpha = 0.1) {
 
+   # get attributes   
+   attrs = mda.getattr(coeffs.values)
+   exclvars = attrs$exclrows
+   nexclvar = length(exclvars)
+
+   # get dimensions and set t-value
+   nvar = dim(ci.coeffs)[1]
+   ncomp = dim(ci.coeffs)[2]
+   ny = dim(ci.coeffs)[3]
+   nobj = dim(ci.coeffs)[4]
    t = qt(1 - ci.alpha/2, nobj - 1)
-
+ 
+   # set up matrices and calculate statistics 
    ci = array(0, dim = c(nvar, ncomp, ny, 2))
    t.values = array(0, dim = c(nvar, ncomp, ny))
    p.values = array(0, dim = c(nvar, ncomp, ny))
-   for (y in 1:ny)
-   {
-      for (comp in 1:ncomp)
-      {
+   for (y in 1:ny) {
+      for (comp in 1:ncomp) {
          coeffs = ci.coeffs[, comp, y, ]
          m = apply(coeffs, 1, mean)
          ssq = apply(t(scale(t(coeffs), center = m, scale = FALSE))^2, 1, sum)
@@ -85,15 +86,34 @@ regcoeffs.getStat = function(obj, ci.coeffs, ci.alpha = 0.1)
          p.values[, comp, y] = 2 * pt(tmin, nobj - 1)
       }   
    }   
+
+   if (nexclvar > 0) {
+      ci.out = array(0, dim = c(nvar + nexclvar, ncomp, ny, 2))
+      t.values.out = array(0, dim = c(nvar + nexclvar, ncomp, ny))
+      p.values.out = array(0, dim = c(nvar + nexclvar, ncomp, ny))
+      ci.out[-exclvars, , ,] = ci
+      t.values.out[-exclvars, , ] = t.values
+      p.values.out[-exclvars, , ] = p.values
+   } else {
+      ci.out = ci
+      t.values.out = t.values
+      p.values.out = p.values
+   }  
    
-   dimnames(t.values) = dimnames(obj)
-   dimnames(p.values) = dimnames(obj)
+   dimnames(t.values.out) = dimnames(p.values.out) = dimnames(coeffs.values)
+   dimnames(ci.out) = c(dimnames(coeffs.values), list(c('Lo', 'Up')))
+   t.values.out = mda.setattr(t.values.out, attrs)
+   p.values.out = mda.setattr(p.values.out, attrs)
+   ci.out = mda.setattr(ci.out, attrs)
+   attr(t.values.out, 'name') = 't-values (Jack-knife)'
+   attr(p.values.out, 'name') = 'p-values (Jack-knife)'
+   attr(t.values.out, 'name') = sprintf('%d%% confidence interval', round((1 - ci.alpha)*100))
    
    stat = list(
-      ci = ci,
-      t.values = t.values,
-      p.values = p.values
-      )
+      ci = ci.out,
+      t.values = t.values.out,
+      p.values = p.values.out
+   )
    
    stat
 }
@@ -113,8 +133,7 @@ regcoeffs.getStat = function(obj, ci.coeffs, ci.alpha = 0.1)
 #' other arguments
 #' 
 #' @export
-as.matrix.regcoeffs = function(x, ncomp = 1, ny = 1, ...)
-{
+as.matrix.regcoeffs = function(x, ncomp = 1, ny = 1, ...) {
    return (x$values[, ncomp, ny, drop = F])
 }
 
@@ -135,12 +154,19 @@ as.matrix.regcoeffs = function(x, ncomp = 1, ny = 1, ...)
 #' other arguments
 #' 
 #' @export
-print.regcoeffs = function(x, ncomp = 1, ny = 1, digits = 3, ...)
-{
+print.regcoeffs = function(x, ncomp = 1, ny = 1, digits = 3, ...) {
    obj = x
    
-   cat('\nRegression coefficients (class plsregcoeffs)\n')
-   print(round(obj$values[, ncomp, ny, drop = F], digits))
+   cat('\nRegression coefficients (class regcoeffs)\n')
+   cat('\nCall:\n')
+   print(obj$call)
+   cat('\nMajor fields:\n')
+   cat('$values - array with regression coefficients\n')
+   cat('$ci - matrix with confidence intervals\n')
+   cat('$t - vector with t-values\n')
+   cat('$p - vector with p-values\n')
+   cat('$alpha - significance level used to calculate the coefficients\n')
+   cat('\nThe last four fields available only if Jack-Knife was used.\n')
 }   
 
 #' Regression coefficients plot
@@ -160,8 +186,6 @@ print.regcoeffs = function(x, ncomp = 1, ny = 1, digits = 3, ...)
 #' vector with colors for the plot (vector or one value)
 #' @param main
 #' main plot title
-#' @param xlab
-#' label for x axis
 #' @param ylab
 #' label for y axis
 #' @param show.line
@@ -172,66 +196,59 @@ print.regcoeffs = function(x, ncomp = 1, ny = 1, digits = 3, ...)
 #' other arguments
 #' 
 #' @export
-plot.regcoeffs = function(x, ncomp = 1, ny = 1, type = NULL, col = NULL, 
-                          main = 'Regression coefficients',
-                          xlab = 'Variables', ylab = 'Coefficients', show.line = T, 
-                          show.ci = T, ...)
-{
+plot.regcoeffs = function(x, ncomp = 1, ny = 1, type = NULL, col = NULL, main = NULL, ylab = NULL, 
+                          show.line = T, show.ci = T, ...) {
+
    obj = x
+   attrs = mda.getattr(obj$values)
    
-   coeffs = obj$values[, ncomp, ny, drop = F]
-   ncoeff = length(coeffs)
+   if (is.null(main)) {   
+      if (is.null(ncomp))
+         main = 'Regression coefficients'
+      else
+         main = sprintf('Regression coefficients (ncomp = %d)', ncomp)
+   }
+   
+   if (is.null(ylab)) {   
+      if (dim(obj$values)[3] == 1 || is.null(dimnames(obj$values)[[3]]))
+         ylab = 'Coefficients'
+      else
+         ylab = sprintf('Coefficients (%s)', dimnames(obj$values)[[3]][ny])
+   }
    
    if (show.line == T)
       show.line = c(NA, 0)
+   ncoeff = nrow(obj$values)
    
-   if (is.null(type))
-   {   
+   if (is.null(type)) {   
       if (ncoeff < 30)
          type = 'b'
       else
          type = 'l'
    }
   
-   data = cbind(1:ncoeff, coeffs)
-   if (show.ci == F || is.null(obj$ci))
-   {   
-      rownames(data) = rownames(obj$values)
-      mdaplot(data, type = type, main = main, xlab = xlab, ylab = ylab, 
-              show.grid = T, show.lines = show.line, ...)
-   }
-   else
-   {
-      ci = obj$ci[, ncomp, ny, ]
-      rownames(data) = rownames(obj$values)
+   data = matrix(obj$values[, ncomp, ny, drop = F], ncol = 1)
+   data = mda.setattr(data, attrs)
+   rownames(data) = rownames(obj$values)
+   if (show.ci == TRUE && !is.null(obj$ci)) {   
+      ci.col = mdaplot.getColors(1)
+      main.col   = 'lightgray'
+      err.margin = matrix(obj$ci[, ncomp, ny, 2], ncol = 1) - data
+      err.maring = mda.setattr(err.margin, attrs)
+      attr(data, 'name') = 'Regression coefficients'      
       
       if (type == 'l')
-      {   
-         if (is.null(col))
-         {
-            cc = mdaplot.getColors(ngroups = 2)
-            cg = mdaplot.getColors(ngroups = 4, colmap = 'gray')
-            col = c(cg[1], cc[1], cc[1])            
-         }
-         
-         type = c('l', 'l', 'l')
-         mdata = cbind(1:ncoeff, coeffs, ci[, 1], ci[, 2])
-      }   
-      else
-      {
-         mdata = list()
-         mdata[[1]] = data 
-         mdata[[2]] = cbind(1:ncoeff, coeffs, ci[, 1], ci[, 2])         
-         type = c(type, 'e')
-
-         if (is.null(col))
-         {
-            cc = mdaplot.getColors(ngroups = 2)
-            cg = mdaplot.getColors(ngroups = 4, colmap = 'gray')
-            col = c(cg[1], cc[1])            
-         }   
-      }   
-      mdaplotg(mdata, type = type, main = main, xlab = xlab, ylab = ylab, 
-              show.grid = T, show.lines = show.line, colmap = col, ...)
-   }   
+         mdaplotg(list(data, data + err.margin, data - err.margin), type = c('l', 'l', 'l'), 
+                  main = main, ylab = ylab, show.legend = F, colmap = c(main.col, ci.col, ci.col), 
+                  show.grid = T, show.lines = show.line, ...)
+     else
+        mdaplotg(list(data, mda.t(mda.cbind(data, err.margin))), type = c(type, 'e'), 
+                 main = main, ylab = ylab, show.legend = F, 
+                 colmap = c(main.col, ci.col), show.grid = T, show.lines = show.line, ...)
+      
+   } else {
+      main.col = ifelse(is.null(col), mdaplot.getColors(1), col)
+      mdaplot(data, type = type, show.grid = T, main = main, ylab = ylab, show.lines = show.line, 
+              col = main.col, ...)
+   }
 }   
